"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.symbolToType = exports.assignableObjectPairs = exports.single = void 0;
const tsutils_1 = require("tsutils");
const total_functions_1 = require("total-functions");
exports.single = (as) => as.length === 1 ? total_functions_1.get(as, 0) : undefined;
/**
 * Throws away non-object types (string, number, boolean, etc) because we don't check those for readonly -> mutable assignment
 * and returns an array of pairs of types that are assignable.
 */
exports.assignableObjectPairs = (rawDestinationType, rawSourceType, checker) => {
    const isAssignableTo = checker.isTypeAssignableTo;
    // eslint-disable-next-line functional/no-conditional-statement
    if (isAssignableTo === undefined) {
        return [];
    }
    const destinationTypeParts = tsutils_1.unionTypeParts(rawDestinationType).filter((t) => tsutils_1.isObjectType(t));
    const sourceTypeParts = tsutils_1.unionTypeParts(rawSourceType).filter((t) => tsutils_1.isObjectType(t));
    return sourceTypeParts.flatMap((sourceTypePart) => destinationTypeParts
        .filter((destinationTypePart) => isAssignableTo(sourceTypePart, destinationTypePart))
        .map((destinationTypePart) => ({
        sourceType: sourceTypePart,
        destinationType: destinationTypePart,
    })));
};
exports.symbolToType = (
// eslint-disable-next-line @typescript-eslint/ban-types
s, checker) => {
    const declarations = s.getDeclarations() || [];
    // TODO: How to choose declaration when there are multiple?
    const declaration = exports.single(declarations);
    return declaration !== undefined
        ? checker.getTypeAtLocation(declaration)
        : undefined;
};
